package com.framsticks.communication;

import com.framsticks.util.FramsticksException;
import com.framsticks.util.StateFunctor;

/**
 * @author Piotr Sniegowski
 */
public abstract class StateCallback<C> extends ResponseCallback<C> implements StateFunctor {


	protected StateFunctor wrapped;

	public StateCallback(StateFunctor wrapped) {
		this.wrapped = wrapped;
	}

	public StateCallback() {
		this.wrapped = null;
	}

	protected StateFunctor move() {
		assert wrapped != null;
		StateFunctor result = wrapped;
		wrapped = null;
		return result;
	}

	protected abstract void callImpl();

	@Override
	public final void call() {
		try {
			callImpl();
		} catch (FramsticksException e) {
			handle(e);
			return;
		}
		if (wrapped != null) {
			wrapped.call();
		}
	}

	@Override
	public void handle(FramsticksException exception) {
		if (wrapped != null) {
			wrapped.handle(exception);
			return;
		}
		throw exception;
	}

	@Override
	public void process(Response response) {
		if (response.getOk()) {
			call();
		} else {
			handle(new FramsticksException().msg("non-ok response").arg("comment", response.getComment()));
		}
	}
}
