package com.framsticks.remote;

import java.util.List;

import org.apache.log4j.Logger;

import static com.framsticks.core.InstanceUtils.*;
import com.framsticks.communication.EventCallback;
import com.framsticks.communication.File;
import com.framsticks.communication.util.LoggingSubscriptionCallback;
import com.framsticks.core.Instance;
import com.framsticks.core.InstanceUtils;
import com.framsticks.core.Path;
import com.framsticks.params.annotations.FramsClassAnnotation;
import com.framsticks.params.types.EventParam;
import com.framsticks.util.Logging;
import com.framsticks.util.PeriodicTask;
import com.framsticks.util.UnaryFunctor;
import com.framsticks.util.UnaryListenersSet;
import com.framsticks.util.dispatching.Future;
import com.framsticks.util.dispatching.RunAt;

@FramsClassAnnotation
public class SimulatorInstance extends RemoteInstance {

	private final static Logger log = Logger.getLogger(SimulatorInstance.class);

	protected Path simulator;

	/**
	 *
	 */
	public SimulatorInstance() {
		super();
	}

	public void setRunning(final boolean running) {
		assert isActive();
		//simulator.call(simulator.getParam(running ? "start" : "stop", ProcedureParam.class), new LoggingStateCallback(log, (running ? "starting" : "stopping") + " server"));
	}

	protected final UnaryListenersSet<Boolean> simulationRunningListeners = new UnaryListenersSet<Boolean>();

	protected void updateSimulationRunning() {
		assert isActive();
		/*
		fetchValue(simulator, getParam(simulator, "running", Param.class), new StateFunctor() {
			@Override
			public void call(Exception e) {
				if (e != null) {
					log.fatal("failed to query simulator running status: " + e);
					return;
				}

				invokeLater(new Runnable() {
					@Override
					public void run() {
						boolean value = bindAccess(simulator).get("running", Boolean.class);
						log.trace("server running: " + value);
						simulationRunningListeners.call(value);
					}
				});

			}
		});
		 */
	}

	public void addRunningStateListener(UnaryFunctor<Boolean, Boolean> listener) {
		assert isActive();
		simulationRunningListeners.add(listener);
	}
	protected void onProtocolVersionNegotiated() {
		dispatch(new RunAt<Instance>() {
			@Override
			public void run() {
				resolveAndFetch(SimulatorInstance.this, "/simulator", new Future<Path>(Logging.logger(log, "failed to resolve simulator node", SimulatorInstance.this)) {
					@Override
					protected void result(Path path) {
						assert isActive();
						simulator = path;
						fireRun(null);
						log.info("resolved simulator node");

						EventParam param = InstanceUtils.getInfoFromCache(simulator).getParamEntry("running_changed", EventParam.class);

						assert param != null;
						connection.subscribe(simulator.getTextual() + "/" + param.getId(), SimulatorInstance.this, new LoggingSubscriptionCallback<Instance>(log, "server running state change", new EventCallback() {
							@Override
							public void call(List<File> files) {
								dispatch(new RunAt<Instance>() {
									@Override
									public void run() {
										updateSimulationRunning();
									}
								});
							}
						}));
						new PeriodicTask<Instance>(SimulatorInstance.this, 1000) {
							@Override
							public void run() {
								updateSimulationRunning();
								again();
							}
						};
					}
				});
			}
		});
	}

}
