package com.framsticks.util.dispatching;

import com.framsticks.util.FramsticksException;

/**
 * @author Piotr Sniegowski
 */
public abstract class Future<T> implements ExceptionResultHandler {

	protected final ExceptionResultHandler handler;

	public Future(ExceptionResultHandler handler) {
		this.handler = handler;
	}

	public Future() {
		this.handler = null;
	}

	protected abstract void result(T result);

	public final void pass(T result) {
		try {
			result(result);
		} catch (FramsticksException e) {
			handle(e);
		}
	}

	public static <T> void passOrHandle(Future<T> future, T value, FramsticksException e) {
		if (e != null) {
			future.handle(e);
		} else {
			future.pass(value);
		}
	}

	@Override
	public void handle(FramsticksException exception) {
		if (handler != null) {
			handler.handle(exception);
			return;
		}
		throw exception;
	}
}
