package com.framsticks.gui;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.fest.swing.core.Robot;
import org.fest.swing.edt.FailOnThreadViolationRepaintManager;
import org.fest.swing.edt.GuiQuery;
import org.fest.swing.fixture.FrameFixture;
import org.fest.swing.fixture.JTreeFixture;
import org.testng.annotations.Test;

import com.framsticks.test.MonitorBasedTest;
import com.framsticks.util.dispatching.Joinable;

import static org.fest.assertions.Assertions.assertThat;
import static org.fest.swing.edt.GuiActionRunner.executeInEDT;
import static org.fest.swing.core.BasicRobot.robotWithNewAwtHierarchy;
import org.fest.swing.edt.GuiActionRunner;

@Test
public abstract class BrowserBaseTest extends MonitorBasedTest {

	private static final Logger log = Logger.getLogger(BrowserTest.class);

	protected Browser browser;
	protected static Robot robot;
	protected FrameFixture frame;
	protected JTreeFixture tree;

	static {
		FailOnThreadViolationRepaintManager.install();
		assertThat(executeInEDT()).isTrue();

		robot = robotWithNewAwtHierarchy();
	}

	@Override
	protected Joinable createSubject() {
		configureBrowser();
		return browser;
	}

	protected void setUpAfterUse() {
		frame = new FrameFixture(robot,
				GuiActionRunner.execute(new GuiQuery<JFrame>() {
					@Override
					protected JFrame executeInEDT() throws Throwable {
						return browser.getMainFrame().getSwing();
					}
				}));

		tree = frame.tree("tree");

		log.debug("frame fixture done");
	}


	protected abstract void configureBrowser();

	protected void clickAndExpandPath(String path) {
		tree.clickPath(path).expandPath(path);
		robot.waitForIdle();
	}

	protected void waitForIdle() {
		robot.waitForIdle();
	}

}
